@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
  *
  * xlogstats.h
- *		Definitions for WAL Statitstics
+ *		Definitions for WAL Statistics
  *
- * Copyright (c) 2022-2024, PostgreSQL Global Development Group
+ * Copyright (c) 2022-2025, PostgreSQL Global Development Group
  *
  * IDENTIFICATION
  *		src/include/access/xlogstats.h
