@@ -3,7 +3,7 @@
  *
  * Prototypes for functions in commands/seclabel.c
  *
- * Portions Copyright (c) 1996-2020, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2025, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  */
 #ifndef SECLABEL_H
@@ -28,7 +28,7 @@ extern ObjectAddress ExecSecLabelStmt(SecLabelStmt *stmt);
 
 typedef void (*check_object_relabel_type) (const ObjectAddress *object,
 										   const char *seclabel);
-extern void register_label_provider(const char *provider,
+extern void register_label_provider(const char *provider_name,
 									check_object_relabel_type hook);
 
 #endif							/* SECLABEL_H */
