@@ -4,10 +4,10 @@
  *	  prototypes for nodeGroup.c
  *
  *
- * Portions Copyright (c) 1996-2006, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2025, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $PostgreSQL: pgsql/src/include/executor/nodeGroup.h,v 1.30 2006/03/05 15:58:56 momjian Exp $
+ * src/include/executor/nodeGroup.h
  *
  *-------------------------------------------------------------------------
  */
@@ -16,10 +16,8 @@
 
 #include "nodes/execnodes.h"
 
-extern int	ExecCountSlotsGroup(Group *node);
 extern GroupState *ExecInitGroup(Group *node, EState *estate, int eflags);
-extern TupleTableSlot *ExecGroup(GroupState *node);
 extern void ExecEndGroup(GroupState *node);
-extern void ExecReScanGroup(GroupState *node, ExprContext *exprCtxt);
+extern void ExecReScanGroup(GroupState *node);
 
-#endif   /* NODEGROUP_H */
+#endif							/* NODEGROUP_H */
