@@ -4,10 +4,10 @@
  *
  *
  *
- * Portions Copyright (c) 1996-2006, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2025, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $PostgreSQL: pgsql/src/include/executor/nodeUnique.h,v 1.22 2006/03/05 15:58:56 momjian Exp $
+ * src/include/executor/nodeUnique.h
  *
  *-------------------------------------------------------------------------
  */
@@ -16,10 +16,8 @@
 
 #include "nodes/execnodes.h"
 
-extern int	ExecCountSlotsUnique(Unique *node);
 extern UniqueState *ExecInitUnique(Unique *node, EState *estate, int eflags);
-extern TupleTableSlot *ExecUnique(UniqueState *node);
 extern void ExecEndUnique(UniqueState *node);
-extern void ExecReScanUnique(UniqueState *node, ExprContext *exprCtxt);
+extern void ExecReScanUnique(UniqueState *node);
 
-#endif   /* NODEUNIQUE_H */
+#endif							/* NODEUNIQUE_H */
